package de.fhmuenster.masterthesis.generator;

import java.util.ArrayList;
import java.util.List;

public class FlowElementDefinitionGenerator extends AbstractTestgenerator {

	private static final String FLOW_ELEMENT_DECLARATION = "\tpublic static final String %s = \"%s\";";
	
	private StringBuilder codegeneratorUserTasks = new StringBuilder();
	private StringBuilder codegeneratorManualTasks = new StringBuilder();
	private StringBuilder codegeneratorScriptTasks = new StringBuilder();
	private StringBuilder codegeneratorServiceTasks = new StringBuilder();
	private StringBuilder codegeneratorBusinessRuleTasks = new StringBuilder();
	private StringBuilder codegeneratorGateways = new StringBuilder();
	private StringBuilder codegeneratorEvents = new StringBuilder();
	
	private List<BPMNGenFlowElement> flowElements = new ArrayList<>();
	private String testcollectionPackageName;
	
	public FlowElementDefinitionGenerator(List<BPMNGenFlowElement> flowElements, String testcollectionPackageName) {
		this.flowElements = flowElements;
		this.testcollectionPackageName = testcollectionPackageName;
	}
	
	public void generateCode() {
		addPackageDeclaration(testcollectionPackageName, codegeneratorUserTasks);
		addSeparatorLine(codegeneratorUserTasks);
		addPackageDeclaration(testcollectionPackageName, codegeneratorManualTasks);
		addSeparatorLine(codegeneratorManualTasks);
		addPackageDeclaration(testcollectionPackageName, codegeneratorScriptTasks);
		addSeparatorLine(codegeneratorScriptTasks);
		addPackageDeclaration(testcollectionPackageName, codegeneratorServiceTasks);
		addSeparatorLine(codegeneratorServiceTasks);
		addPackageDeclaration(testcollectionPackageName, codegeneratorBusinessRuleTasks);
		addSeparatorLine(codegeneratorBusinessRuleTasks);
		addPackageDeclaration(testcollectionPackageName, codegeneratorGateways);
		addSeparatorLine(codegeneratorGateways);
		addPackageDeclaration(testcollectionPackageName, codegeneratorEvents);
		addSeparatorLine(codegeneratorEvents);
		
		addClassHeader(NameProvider.USER_TASK_NAME, codegeneratorUserTasks);
		addSeparatorLine(codegeneratorUserTasks);
		addClassHeader(NameProvider.MANUAL_TASK_NAME, codegeneratorManualTasks);
		addSeparatorLine(codegeneratorManualTasks);
		addClassHeader(NameProvider.SCRIPT_TASK_NAME, codegeneratorScriptTasks);
		addSeparatorLine(codegeneratorScriptTasks);
		addClassHeader(NameProvider.SERVICE_TASK_NAME, codegeneratorServiceTasks);
		addSeparatorLine(codegeneratorServiceTasks);
		addClassHeader(NameProvider.BUSINESS_RULE_NAME, codegeneratorBusinessRuleTasks);
		addSeparatorLine(codegeneratorBusinessRuleTasks);
		addClassHeader(NameProvider.GATEWAYS_NAME, codegeneratorGateways);
		addSeparatorLine(codegeneratorGateways);
		addClassHeader(NameProvider.EVENTS_NAME, codegeneratorEvents);
		addSeparatorLine(codegeneratorEvents);
		
		addFlowElementDeclarations(FlowElementType.USER_TASK, codegeneratorUserTasks);
		addSeparatorLine(codegeneratorUserTasks);
		addFlowElementDeclarations(FlowElementType.MANUAL_TASK, codegeneratorManualTasks);
		addSeparatorLine(codegeneratorManualTasks);
		addFlowElementDeclarations(FlowElementType.SCRIPT_TASK, codegeneratorScriptTasks);
		addSeparatorLine(codegeneratorScriptTasks);
		addFlowElementDeclarations(FlowElementType.SERVICE_TASK_DELEGATE, codegeneratorServiceTasks);
		addFlowElementDeclarations(FlowElementType.SERVICE_TASK_EXTERNAL, codegeneratorServiceTasks);
		addSeparatorLine(codegeneratorServiceTasks);
		addFlowElementDeclarations(FlowElementType.BUSINESS_RULE_TASK, codegeneratorBusinessRuleTasks);
		addSeparatorLine(codegeneratorBusinessRuleTasks);
		addFlowElementDeclarations(FlowElementType.GATEWAY, codegeneratorGateways);
		addSeparatorLine(codegeneratorGateways);
		addFlowElementDeclarations(FlowElementType.START_EVENT, codegeneratorEvents);
		addFlowElementDeclarations(FlowElementType.END_EVENT, codegeneratorEvents);
		addSeparatorLine(codegeneratorEvents);
		
		addClassClosing(codegeneratorUserTasks);
		addClassClosing(codegeneratorManualTasks);
		addClassClosing(codegeneratorScriptTasks);
		addClassClosing(codegeneratorServiceTasks);
		addClassClosing(codegeneratorBusinessRuleTasks);
		addClassClosing(codegeneratorGateways);
		addClassClosing(codegeneratorEvents);
	}
	
	private void addFlowElementDeclarations(FlowElementType type, StringBuilder builder) {
		flowElements.stream() //
			.filter(element -> type.equals(element.getType())) //
			.forEach(element -> addFlowElementDeclaration(element.getType(), element.getId(), builder));
	}
	
	private void addFlowElementDeclaration(FlowElementType type, String flowElementName, StringBuilder builder) {
		appendLine(String.format(FLOW_ELEMENT_DECLARATION, NameProvider.getStaticFinalFlowElementName(type, flowElementName), flowElementName), builder);
	}
	
	public String getUserTaskCode() {
		return codegeneratorUserTasks.toString();
	}
	
	public String getManualTaskCode() {
		return codegeneratorManualTasks.toString();
	}
	
	public String getScriptTaskCode() {
		return codegeneratorScriptTasks.toString();
	}
	
	public String getServiceTaskCode() {
		return codegeneratorServiceTasks.toString();
	}
	
	public String getBusinessRuleTaskCode() {
		return codegeneratorBusinessRuleTasks.toString();
	}
	
	public String getGatewayCode() {
		return codegeneratorGateways.toString();
	}
	
	public String getEventCode() {
		return codegeneratorEvents.toString();
	}
}
