package de.fhmuenster.masterthesis.generator;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import de.fhmuenster.masterthesis.generator.mockdata.DelegateMock;
import de.fhmuenster.masterthesis.generator.mockdata.ExternalServiceMock;
import de.fhmuenster.masterthesis.generator.mockdata.HasMockRules;
import de.fhmuenster.masterthesis.generator.mockdata.ManualMock;
import de.fhmuenster.masterthesis.generator.mockdata.MockCompare;
import de.fhmuenster.masterthesis.generator.mockdata.MockDeclaration;
import de.fhmuenster.masterthesis.generator.mockdata.MockRule;
import de.fhmuenster.masterthesis.generator.mockdata.ObjectCompare;
import de.fhmuenster.masterthesis.generator.mockdata.PrimitiveCompare;
import de.fhmuenster.masterthesis.generator.utils.TestgeneratorClassUtils;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.BooleanVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.Compare;
import de.fhmuenster.masterthesis.testgeneratorDSL.DelegateServiceTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.IntVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.MockingRule;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringCompare;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringCompareSymbol;
import de.fhmuenster.masterthesis.testgeneratorDSL.StringVariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclaration;
import de.fhmuenster.masterthesis.utils.TestgeneratorDSLUtils;

public class MockCreator {

	public MockCreator() {
	}
	
	public List<DelegateMock> createDelegationMock(List<de.fhmuenster.masterthesis.testgeneratorDSL.DelegateMock> delegateMocks) {
		List<DelegateMock> mocks = new ArrayList<>();
		
		for(de.fhmuenster.masterthesis.testgeneratorDSL.DelegateMock delegateMock : delegateMocks) {
			DelegateServiceTaskFlowElement serviceTaskFlowElement = delegateMock.getServiceTaskReference().getRef();
			String delegateName = delegateMock.getName();
			String delegateFullQualifiedClassName = delegateMock.getDelegateClass();
			String delegateClassName = TestgeneratorClassUtils.getClassName(delegateFullQualifiedClassName);
			String delegateRegistrationName = serviceTaskFlowElement.getDelegate();
			
			DelegateMock mock = new DelegateMock();
			mock.setName(delegateName);
			mock.setPackageName(delegateFullQualifiedClassName);
			mock.setDelegateClassName(delegateClassName);
			mock.setRegistrationName(delegateRegistrationName);
			
			createManualMocks(delegateMock.getManualMocks(), mock);
			
			mocks.add(mock);
		}
		
		return mocks;
	}
	
	private void createManualMocks(EList<de.fhmuenster.masterthesis.testgeneratorDSL.ManualMock> manualMocks, DelegateMock delegateMock) {
		for (de.fhmuenster.masterthesis.testgeneratorDSL.ManualMock manualMock : manualMocks) {
			String packageName = manualMock.getServiceName();
			String className = TestgeneratorClassUtils.getClassName(packageName);
			String setter = manualMock.getSetter();
			
			ManualMock mock = new ManualMock();
			mock.setPackageName(packageName);
			mock.setClassName(className);
			mock.setSetter(setter);
			
			delegateMock.addManualMocks(mock);
		}
	}

	public List<ExternalServiceMock> createExternalServiceMock(List<de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceMock> externalServiceMocks) {
		List<ExternalServiceMock> mocks = new ArrayList<>();
		
		for(de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceMock externalServiceMock : externalServiceMocks) {
			String name = externalServiceMock.getName();
			
			ExternalServiceMock mock = new ExternalServiceMock();
			mock.setName(name);
			mock.setExternalTopic(externalServiceMock.getExternalTopic().getRef().getName());
			
			createMockRules(externalServiceMock.getMockingRules(), mock);
	
			mocks.add(mock);
		}
		
		return mocks;
	}

	private void createMockRules(EList<MockingRule> mockingRules, HasMockRules mock) {
		for(MockingRule rule : mockingRules) {
			MockRule mockRule = new MockRule();
			
			for(VariableDeclaration declaration : rule.getVariables()) {
				MockDeclaration mockDeclaration = new MockDeclaration();

				if(declaration instanceof StringVariableDeclaration) {
					String variableName = ((StringVariableDeclaration) declaration).getKey().getName();
					String variableValue = ((StringVariableDeclaration) declaration).getValue();
					
					mockDeclaration.setVariable(variableName);
					mockDeclaration.setValue(variableValue);
				}
				if(declaration instanceof IntVariableDeclaration) {
					String variableName = ((IntVariableDeclaration) declaration).getKey().getName();
					int variableValue = ((IntVariableDeclaration) declaration).getValue();
					
					mockDeclaration.setVariable(variableName);
					mockDeclaration.setValue(variableValue);
				}
				if(declaration instanceof BooleanVariableDeclaration) {
					String variableName = ((BooleanVariableDeclaration) declaration).getKey().getName();
					Boolean variableValue = TestgeneratorDSLUtils.mapDSLBoolean(((BooleanVariableDeclaration) declaration).getValue());
					
					mockDeclaration.setVariable(variableName);
					mockDeclaration.setValue(variableValue);
				}
				
				mockRule.addDeclaration(mockDeclaration);
			}
			
			for(Compare compare : rule.getComparisons()) {
				MockCompare mockCompare = null;
				
				if(compare instanceof StringCompare) {
					mockCompare = new ObjectCompare();
					
					String variableName = ((StringCompare) compare).getKey().getName();
					String variableValue = ((StringCompare) compare).getValue();
					String literal = ((StringCompare) compare).getCompareSymbol().getLiteral();
					
					((ObjectCompare) mockCompare).setVariable(variableName);
					((ObjectCompare) mockCompare).setValue(variableValue);
					((ObjectCompare) mockCompare).setPositiveCheck(literal.equals(StringCompareSymbol.EQUALS.getLiteral()));
				}
				
				if(compare instanceof IntCompare) {
					mockCompare = new PrimitiveCompare();
					
					String variableName = ((IntCompare) compare).getKey().getName();
					int variableValue = ((IntCompare) compare).getValue();
					String compareOperator = ((IntCompare) compare).getCompareSymbol().getLiteral();
					
					((PrimitiveCompare) mockCompare).setVariable(variableName);
					((PrimitiveCompare) mockCompare).setValue(variableValue);
					((PrimitiveCompare) mockCompare).setCompareOperator(compareOperator);
				}
				
				if(compare instanceof BooleanCompare) {
					mockCompare = new PrimitiveCompare();
					
					String variableName = ((BooleanCompare) compare).getKey().getName();
					Boolean variableValue = TestgeneratorDSLUtils.mapDSLBoolean(((BooleanCompare) compare).getValue());
					String compareOperator = ((BooleanCompare) compare).getCompareSymbol().getLiteral();
					
					((PrimitiveCompare) mockCompare).setVariable(variableName);
					((PrimitiveCompare) mockCompare).setValue(variableValue);
					((PrimitiveCompare) mockCompare).setCompareOperator(compareOperator);
				}
				
				mockRule.addComparison(mockCompare);
			}
			
			mock.addMockRule(mockRule);
		}
	}
}
