package de.fhmuenster.masterthesis.generator.mockdata;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ExternalServiceMock implements IsMock, HasMockRules {

	private List<MockRule> mockRules = new ArrayList<>();
	private String name;
	private String externalTopic;
	
	public ExternalServiceMock() {
	}
	
	@Override
	public List<MockRule> getMockRules() {
		return mockRules;
	}

	@Override
	public void setMockRules(List<MockRule> mockRules) {
		this.mockRules = mockRules;
	}
	
	@Override
	public void addMockRule(MockRule mockRule) {
		this.mockRules.add(mockRule);
	}
	
	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}
	
	public String getExternalTopic() {
		return externalTopic;
	}
	
	public void setExternalTopic(String topic) {
		this.externalTopic = topic;
	}
	
	public List<String> getVariables() {
		return Stream.concat(getReadVariables().stream(), getWriteVariables().stream()) //
			.distinct() //
			.collect(Collectors.toList()); //
	}
	
	public List<String> getReadVariables() {
		return mockRules.stream() //
			.flatMap(rule -> rule.getComparisons().stream()) //
			.map(comparison -> comparison.getVariable()) //
			.distinct() //
			.collect(Collectors.toList()); //
	}
	
	public List<String> getReadVariables(Class<?> dataType) {
		return mockRules.stream() //
			.flatMap(rule -> rule.getComparisons().stream()) //
			.filter(comparison -> comparison.getValue().getClass().equals(dataType)) //
			.map(comparison -> comparison.getVariable()) //
			.distinct() //
			.collect(Collectors.toList()); //
	}
	
	public List<String> getWriteVariables() {
		return mockRules.stream() //
				.flatMap(rule -> rule.getDeclarations().stream()) //
				.map(declaration -> declaration.getVariable()) //
				.distinct() //
				.collect(Collectors.toList()); //
	}
}
